@@ -189,7 +189,7 @@ re_string_construct_common (str, len, pstr, trans, icase, dfa)
   pstr->raw_mbs = (const unsigned char *) str;
   pstr->len = len;
   pstr->raw_len = len;
-  pstr->trans = trans;
+  pstr->trans = (unsigned RE_TRANSLATE_TYPE) trans;
   pstr->icase = icase ? 1 : 0;
   pstr->mbs_allocated = (trans != NULL || icase);
   pstr->mb_cur_max = dfa->mb_cur_max;
@@ -579,7 +579,7 @@ re_string_reconstruct (pstr, idx, eflags)
      int idx, eflags;
 {
   int offset = idx - pstr->raw_mbs_idx;
-  if (offset < 0)
+  if (BE (offset < 0, 0))
     {
       /* Reset buffer.  */
 #ifdef RE_ENABLE_I18N
@@ -599,10 +599,10 @@ re_string_reconstruct (pstr, idx, eflags)
       offset = idx;
     }
 
-  if (offset != 0)
+  if (BE (offset != 0, 1))
     {
       /* Are the characters which are already checked remain?  */
-      if (offset < pstr->valid_raw_len
+      if (BE (offset < pstr->valid_raw_len, 1)
 #ifdef RE_ENABLE_I18N
 	  /* Handling this would enlarge the code too much.
 	     Accept a slowdown in that case.  */
@@ -613,11 +613,11 @@ re_string_reconstruct (pstr, idx, eflags)
 	  /* Yes, move them to the front of the buffer.  */
 	  pstr->tip_context = re_string_context_at (pstr, offset - 1, eflags);
 #ifdef RE_ENABLE_I18N
-	  if (pstr->mb_cur_max > 1)
+	  if (BE (pstr->mb_cur_max, 1) > 1)
 	    memmove (pstr->wcs, pstr->wcs + offset,
 		     (pstr->valid_len - offset) * sizeof (wint_t));
 #endif /* RE_ENABLE_I18N */
-	  if (pstr->mbs_allocated)
+	  if (BE (pstr->mbs_allocated, 0))
 	    memmove (pstr->mbs, pstr->mbs + offset,
 		     pstr->valid_len - offset);
 	  pstr->valid_len -= offset;
@@ -715,7 +715,7 @@ re_string_reconstruct (pstr, idx, eflags)
 				      ? CONTEXT_NEWLINE : 0));
 	    }
 	}
-      if (!pstr->mbs_allocated)
+      if (!BE (pstr->mbs_allocated, 0))
 	pstr->mbs += offset;
     }
   pstr->raw_mbs_idx = idx;
@@ -724,7 +724,7 @@ re_string_reconstruct (pstr, idx, eflags)
 
   /* Then build the buffers.  */
 #ifdef RE_ENABLE_I18N
-  if (pstr->mb_cur_max > 1)
+  if (BE (pstr->mb_cur_max, 1) > 1)
     {
       if (pstr->icase)
 	{
@@ -737,16 +737,17 @@ re_string_reconstruct (pstr, idx, eflags)
     }
   else
 #endif /* RE_ENABLE_I18N */
+  if (BE (pstr->mbs_allocated, 0))
     {
       if (pstr->icase)
 	build_upper_buffer (pstr);
       else if (pstr->trans != NULL)
 	re_string_translate_buffer (pstr);
-      else
-	pstr->valid_len = pstr->len;
     }
-  pstr->cur_idx = 0;
+  else
+    pstr->valid_len = pstr->len;
 
+  pstr->cur_idx = 0;
   return REG_NOERROR;
 }
 
@@ -758,7 +759,7 @@ re_string_peek_byte_case (pstr, idx)
   int ch, off;
 
   /* Handle the common (easiest) cases first.  */
-  if (BE (!pstr->icase, 1))
+  if (BE (!pstr->mbs_allocated, 1))
     return re_string_peek_byte (pstr, idx);
 
 #ifdef RE_ENABLE_I18N
@@ -774,8 +775,6 @@ re_string_peek_byte_case (pstr, idx)
 #endif
 
   ch = pstr->raw_mbs[pstr->raw_mbs_idx + off];
-  if (pstr->trans)
-    ch = pstr->trans[ch];
 
 #ifdef RE_ENABLE_I18N
   /* Ensure that e.g. for tr_TR.UTF-8 BACKSLASH DOTLESS SMALL LETTER I
@@ -793,15 +792,13 @@ static unsigned char
 re_string_fetch_byte_case (pstr)
      re_string_t *pstr;
 {
-  int ch;
-
-  if (BE (!pstr->icase, 1))
+  if (BE (!pstr->mbs_allocated, 1))
     return re_string_fetch_byte (pstr);
 
 #ifdef RE_ENABLE_I18N
   if (pstr->offsets_needed)
     {
-      int off;
+      int off, ch;
 
       /* For tr_TR.UTF-8 [[:islower:]] there is
 	 [[: CAPITAL LETTER I WITH DOT lower:]] in mbs.  Skip
@@ -815,8 +812,6 @@ re_string_fetch_byte_case (pstr)
 
       off = pstr->offsets[pstr->cur_idx];
       ch = pstr->raw_mbs[pstr->raw_mbs_idx + off];
-      if (pstr->trans)
-	ch = pstr->trans[ch];
 
       if (! isascii (ch))
 	return re_string_fetch_byte (pstr);
@@ -827,10 +822,7 @@ re_string_fetch_byte_case (pstr)
     }
 #endif
 
-  ch = pstr->raw_mbs[pstr->raw_mbs_idx + pstr->cur_idx++];
-  if (pstr->trans)
-    ch = pstr->trans[ch];
-  return ch;
+  return pstr->raw_mbs[pstr->raw_mbs_idx + pstr->cur_idx++];
 }
 
 static void
@@ -853,18 +845,17 @@ re_string_context_at (input, idx, eflags)
      int idx, eflags;
 {
   int c;
-  if (idx < 0 || idx == input->len)
-    {
-      if (idx < 0)
-	/* In this case, we use the value stored in input->tip_context,
-	   since we can't know the character in input->mbs[-1] here.  */
-	return input->tip_context;
-      else /* (idx == input->len) */
-	return ((eflags & REG_NOTEOL) ? CONTEXT_ENDBUF
-		: CONTEXT_NEWLINE | CONTEXT_ENDBUF);
-    }
+  if (BE (idx < 0, 0))
+    /* In this case, we use the value stored in input->tip_context,
+       since we can't know the character in input->mbs[-1] here.  */
+    return input->tip_context;
+
+  else if (BE (idx == input->len, 0))
+    return ((eflags & REG_NOTEOL) ? CONTEXT_ENDBUF
+	    : CONTEXT_NEWLINE | CONTEXT_ENDBUF);
+
 #ifdef RE_ENABLE_I18N
-  if (input->mb_cur_max > 1)
+  else if (BE (input->mb_cur_max, 1) > 1)
     {
       wint_t wc;
       int wc_idx = idx;
@@ -1657,5 +1648,6 @@ free_state (state)
     }
   re_node_set_free (&state->nodes);
   re_free (state->trtable);
+  re_free (state->word_trtable);
   re_free (state);
 }
