@@ -195,7 +195,7 @@ bad_command(ch)
   char ch;
 {
   const char *msg = _(UNKNOWN_CMD);
-  char *unknown_cmd = xmalloc(strlen(msg) - 1);
+  char *unknown_cmd = xmalloc(strlen(msg));
   sprintf(unknown_cmd, msg, ch);
   bad_prog(unknown_cmd);
 }
@@ -511,7 +511,7 @@ snarf_char_class(b, cur_stat)
 	  if (pending_mb)
 	    continue;
 
-	  if (state == 0)
+	  if (state == 0 || state == 1)
 	    return ch;
 	  else if (state == 3)
 	    state = 0;
@@ -1318,7 +1318,7 @@ compile_program(vector)
                 cur_cmd->x.translatemb = trans_pairs;
                 for (i = 0; i < src_char_num; i++)
                   {
-                    if (idx >= len)
+                    if (idx >= dest_len)
                       bad_prog(_(Y_CMD_LEN));
 
                     /* Set the i-th source character.  */
@@ -1328,7 +1328,7 @@ compile_program(vector)
                     src_buf += src_lens[i]; /* Forward to next character.  */
 
                     /* Fetch the i-th destination character.  */
-                    mbclen = MBRLEN (dest_buf + idx, len - idx, &cur_stat);
+                    mbclen = MBRLEN (dest_buf + idx, dest_len - idx, &cur_stat);
                     /* An invalid sequence, or a truncated multibyte character.
                        We treat it as a singlebyte character.  */
                     if (mbclen == (size_t) -1 || mbclen == (size_t) -2
@@ -1342,7 +1342,7 @@ compile_program(vector)
                     idx += mbclen; /* Forward to next character.  */
                   }
                 trans_pairs[2 * i] = NULL;
-                if (idx != len)
+                if (idx != dest_len)
                   bad_prog(_(Y_CMD_LEN));
               }
             else
