@@ -638,7 +638,7 @@ open_next_file(name, input)
 {
   buffer.length = 0;
 
-  if (name[0] == '-' && name[1] == '\0')
+  if (name[0] == '-' && name[1] == '\0' && !in_place_extension)
     {
       clearerr(stdin);	/* clear any stale EOF indication */
       input->fp = stdin;
@@ -667,10 +667,22 @@ open_next_file(name, input)
 	strcpy(tmpdir, ".");
       
       input->in_file_name = name;
-      output_file.fp = ck_mkstemp (&input->out_file_name, tmpdir, "sed");
-      free (tmpdir);
+
+      if (isatty (fileno (input->fp)))
+        panic(_("couldn't edit %s: is a terminal"), input->in_file_name);
 
       fstat (fileno (input->fp), &st);
+      if (!S_ISREG (st.st_mode))
+        panic(_("couldn't edit %s: not a regular file"), input->in_file_name);
+
+      output_file.fp = ck_mkstemp (&input->out_file_name, tmpdir, "sed");
+      output_file.missing_newline = false;
+      free (tmpdir);
+
+      if (!output_file.fp)
+        panic(_("couldn't open temporary file %s: %s"), input->out_file_name, strerror(errno));
+
+      output_fd = fileno (output_file.fp);
 #ifdef HAVE_FCHMOD
       fchmod (output_fd, st.st_mode);
 #endif
@@ -678,11 +690,6 @@ open_next_file(name, input)
       if (fchown (output_fd, st.st_uid, st.st_gid) == -1)
         fchown (output_fd, -1, st.st_gid);
 #endif
-
-      output_file.missing_newline = false;
-
-      if (!output_file.fp)
-        panic(_("Couldn't open temporary file %s: %s"), input->out_file_name, strerror(errno));
     }
   else
     output_file.fp = stdout;
@@ -860,7 +867,7 @@ match_an_address_p(addr, input)
       /* ADDR_IS_NUM is handled in match_address_p.  */
     case ADDR_IS_NUM:
     default:
-      panic(_("INTERNAL ERROR: bad address type"));
+      panic("INTERNAL ERROR: bad address type");
     }
   /*NOTREACHED*/
   return false;
@@ -1619,7 +1626,7 @@ execute_program(vec, input)
 	      break;
 
 	    default:
-	      panic(_("INTERNAL ERROR: Bad cmd %c"), cur_cmd->cmd);
+	      panic("INTERNAL ERROR: Bad cmd %c", cur_cmd->cmd);
 	    }
 	}
 
